import asyncio
import os
import random
import string
from getpass import getpass

from colorama import init, Fore, Style

from pynubankasync import NuException
from pynubankasync.utils.certificate_generator import CertificateGenerator


def generate_random_id() -> str:
    return "".join(random.choices(string.ascii_lowercase + string.digits, k=12))


def log(message, color=Fore.BLUE):
    print(f"{color}{Style.DIM}[*] {Style.NORMAL}{Fore.LIGHTBLUE_EX}{message}")


def save_cert(cert, name):
    path = os.path.join(os.getcwd(), name)
    with open(path, "wb") as cert_file:
        cert_file.write(cert.export())


async def main():
    init()

    log(
        f"Starting {Fore.MAGENTA}{Style.DIM}PyNubank{Style.NORMAL}{Fore.LIGHTBLUE_EX} context creation."
    )

    device_id = generate_random_id()

    log(f"Generated random id: {device_id}")

    cpf = input(f"[>] Enter your CPF(Numbers only): ")
    password = getpass("[>] Enter your password (Used on the app/website): ")

    generator = CertificateGenerator(cpf, password, device_id)
    await generator.start()

    log("Requesting e-mail code")
    try:
        email = await generator.request_code()
    except NuException:
        log(f"{Fore.RED}Failed to request code. Check your credentials!", Fore.RED)
        return

    log(f"Email sent to {Fore.LIGHTBLACK_EX}{email}{Fore.LIGHTBLUE_EX}")
    code = input("[>] Type the code received by email: ")

    cert1, cert2 = generator.exchange_certs(code)

    save_cert(cert1, "cert.p12")

    print(f"{Fore.GREEN}Certificates generated successfully. (cert.pem)")
    print(
        f"{Fore.YELLOW}Warning, keep these certificates safe (Do not share or version in git)"
    )


if __name__ == "__main__":
    asyncio.run(main())
